"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureFunctionExplorer = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const prefix_1 = require("./decorators/prefix");
class AzureFunctionExplorer {
    static servicesOf(app) {
        const modules = [...app.getModules().values()].map((module) => module);
        return modules.flatMap((module) => [
            ...module.providers.values()
        ]);
    }
    static servicesWithMethods(services) {
        return services.map(({ instance }) => {
            if (!instance) {
                return [instance, []];
            }
            const prototype = Object.getPrototypeOf(instance);
            const methods = AzureFunctionExplorer.metadataScanner.getAllFilteredMethodNames(prototype);
            return [instance, [...methods]];
        });
    }
    static servicesWithTrigger(servicesWithMethods) {
        return servicesWithMethods
            .map(([instance, methods]) => [
            instance,
            methods.filter((method) => {
                return Reflect.hasMetadata(prefix_1.TRIGGER_DECORATOR_KEY, instance[method]);
            })
        ])
            .filter(([, methods]) => methods.length);
    }
    static servicesWithTriggerForFunction(servicesWithMethods, functionName) {
        return servicesWithMethods
            .map(([instance, methods]) => [
            instance,
            methods.filter((method) => Reflect.getMetadata(prefix_1.TRIGGER_DECORATOR_KEY, instance[method]) ===
                functionName)
        ])
            .filter(([, methods]) => methods.length);
    }
    static callWithContext([service, method], context, ...rest) {
        return __awaiter(this, void 0, void 0, function* () {
            const injections = Reflect.getMetadata(prefix_1.CONTEXT_DECORATOR_KEY, service, method);
            const actualMethod = service[method];
            if (!injections) {
                return yield actualMethod.apply(service, []);
            }
            const injectionMappings = Object.entries(injections);
            const args = injectionMappings.reduce((newArgs, [_, shouldInjectContext]) => {
                return shouldInjectContext
                    ? [...newArgs, context, ...rest]
                    : [...newArgs, undefined, ...rest];
            }, []);
            return yield actualMethod.apply(service, args);
        });
    }
    static dispatchFunctionTrigger(app, context, ...rest) {
        return __awaiter(this, void 0, void 0, function* () {
            const container = app.container;
            const services = AzureFunctionExplorer.servicesOf(container);
            const servicesWithMethods = this.servicesWithMethods(services);
            const servicesWithTimedTriggerMethods = this.servicesWithTriggerForFunction(servicesWithMethods, context.executionContext.functionName);
            const methodCalls = servicesWithTimedTriggerMethods.flatMap(([service, methods]) => {
                return methods.map((method) => [service, method]);
            });
            yield Promise.all(methodCalls.map(([service, method]) => __awaiter(this, void 0, void 0, function* () {
                this.logger.log(`Triggering ${service.constructor.name}.${method}`);
                context.log(`Triggering ${service.constructor.name}.${method}`);
                yield this.callWithContext([service, method], context, ...rest);
            })));
        });
    }
}
exports.AzureFunctionExplorer = AzureFunctionExplorer;
AzureFunctionExplorer.logger = new common_1.Logger(AzureFunctionExplorer.name);
AzureFunctionExplorer.metadataScanner = new core_1.MetadataScanner();
